
from zeam.form.base.interfaces import IField
from zeam.form.base.widgets import DisplayFieldWidget
from zeam.form.base.markers import ModeMarker

from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.traversing.browser.interfaces import IAbsoluteURL

from grokcore import component as grok


class LinkFieldWidget(DisplayFieldWidget):
    grok.adapts(IField, Interface, Interface)
    grok.name('link')

    def url(self):
        context = self.form.context
        return getMultiAdapter((context, self.request), IAbsoluteURL)()


LINK = ModeMarker('LINK', extractable=False)




# URI widget

import re

from zeam.form.base.markers import Marker, NO_VALUE
from zeam.form.base.widgets import FieldWidget
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


isURI = re.compile(
    # scheme
    r"[a-zA-z0-9+.-]+:"
    # non space (should be pickier)
    r"\S*$").match


class URIField(Field):
    """A text line field.
    """
    target = '_self'

    def __init__(self, title,
                 minLength=0,
                 maxLength=None,
                 **options):
        super(URIField, self).__init__(title, **options)
        self.minLength = minLength
        self.maxLength = maxLength

    def isEmpty(self, value):
        return value is NO_VALUE or not len(value)

    def validate(self, value, form):
        error = super(URIField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker) and len(value):
            assert isinstance(value, basestring)
            if not isURI(value):
                return _(u"The URI is malformed.")
            if self.minLength and len(value) < self.minLength:
                return _(u"The URI is too short.")
            if self.maxLength and len(value) > self.maxLength:
                return _(u"The URI is too long.")
        return None

# BBB
URISchemaField = URIField

class URIWidget(FieldWidget):
    grok.adapts(URIField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-uri']
    defaultHtmlAttributes = set(['readonly', 'required', 'autocomplete',
                                 'maxlength', 'pattern', 'placeholder',
                                 'size', 'style'])


class URIDisplayWidget(FieldWidget):
    grok.adapts(URIField, Interface, Interface)
    grok.name('display')

    @property
    def target(self):
        return self.component.target


def URISchemaFactory(schema):
    field = URIField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        minLength=schema.min_length,
        maxLength=schema.max_length,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def register():
    registerSchemaField(URISchemaFactory, schema_interfaces.IURI)



# -*- coding: utf-8 -*-

try:
    import hashlib
    md5hash = lambda s: hashlib.md5(s).hexdigest()
except ImportError:
    import md5
    md5hash = lambda s: md5.new(s).hexdigest()

try:
    # If you have the fanstatic extra, add support to include widget JS
    import fanstatic
    import js.jquery
    import zeam.jsontemplate

    library = fanstatic.Library('zeam.form.ztk.widgets', 'static')
    collection = fanstatic.Resource(
        library, 'collection.js', depends=[js.jquery.jquery,
                                           zeam.jsontemplate.jsontemplate])
    requireCollectionResources = collection.need
except ImportError:
    class Library(object):
        name = ""

        def init_library_nr(self):
            pass

        def library_nr(self):
            return 20000

        @property
        def known_assets(self):
            return []


    library = Library()
    requireCollectionResources = lambda: None

from zeam.form.base.datamanager import NoneDataManager
from zeam.form.base.errors import Errors, Error
from zeam.form.base.fields import Fields
from zeam.form.base.markers import Marker
from zeam.form.base.form import cloneFormData
from zeam.form.base.interfaces import IField, IWidget, IWidgetExtractor
from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import WidgetExtractor, FieldWidget, Widgets
from zeam.form.ztk.fields import Field, registerSchemaField
from zeam.form.ztk.interfaces import ICollectionField, IListField
from zeam.form.ztk.widgets.choice import ChoiceField, ChoiceFieldWidget
from zeam.form.ztk.widgets.object import ObjectField

from grokcore import component as grok
from zope import component
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


class CollectionField(Field):
    """A collection field.
    """
    grok.implements(ICollectionField)

    collectionType = list
    allowAdding = True
    allowRemove = True
    inlineValidation = False

    def __init__(self, title,
                 valueField=None,
                 minLength=0,
                 maxLength=None,
                 **options):
        super(CollectionField, self).__init__(title, **options)
        self._valueField = IField(valueField, None)
        self.minLength = minLength
        self.maxLength = maxLength

    @property
    def valueField(self):
        return self._valueField

    def validate(self, value, form):
        error = super(CollectionField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker):
            assert isinstance(value, self.collectionType)
            if self.minLength and len(value) < self.minLength:
                return _(u"There are too few items.")
            if self.maxLength and len(value) > self.maxLength:
                return _(u"There are too many items.")
        return None

    def isEmpty(self, value):
        return value is NO_VALUE or not len(value)


# BBB
CollectionSchemaField = CollectionField


class ListField(CollectionField):
    """A list field
    """
    grok.implements(IListField)
    collectionType = list
    allowOrdering = True


# BBB
ListSchemaField = ListField


class SetField(CollectionField):
    """A set field
    """
    collectionType = set


# BBB
SetSchemaField = SetField


class TupleField(CollectionField):
    """A tuple field.
    """
    collectionType = tuple


# BBB
TupleSchemaField = TupleField


def newCollectionWidgetFactory(mode=u"", interface=IWidget):
    def collectionWidgetFactory(field, form, request):
        """A widget of a collection is a bit advanced. We have to adapt
        the sub-type of the field as well.
        """
        widget = component.getMultiAdapter(
            (field, field.valueField, form, request), interface, name=mode)
        return widget
    return collectionWidgetFactory


grok.global_adapter(
    newCollectionWidgetFactory(mode='input'),
    adapts=(ICollectionField, Interface, Interface),
    provides=IWidget,
    name='input')

grok.global_adapter(
    newCollectionWidgetFactory(mode='input-list'),
    adapts=(ICollectionField, Interface, Interface),
    provides=IWidget,
    name='input-list')

grok.global_adapter(
    newCollectionWidgetFactory(mode='display'),
    adapts=(ICollectionField, Interface, Interface),
    provides=IWidget,
    name='display')

grok.global_adapter(
    newCollectionWidgetFactory(interface=IWidgetExtractor),
    adapts=(ICollectionField, Interface, Interface),
    provides=IWidgetExtractor)


class MultiGenericFieldWidget(FieldWidget):
    grok.adapts(ICollectionField, Interface, Interface, Interface)

    allowAdding = True
    allowRemove = True
    inlineValidation = False

    def __init__(self, field, value_field, form, request):
        super(MultiGenericFieldWidget, self).__init__(field, form, request)
        self.allowAdding = field.allowAdding
        self.allowRemove = field.allowRemove
        self.inlineValidation = field.inlineValidation
        self.valueField = value_field
        self.valueWidgets = Widgets()
        self.haveValues = False

    def createValueWidget(self, new_identifier, value):
        field = self.valueField.clone(new_identifier=str(new_identifier))
        form = cloneFormData(self.form, prefix=self.identifier)
        if value is not None:
            form.ignoreContent = False
            form.setContentData(NoneDataManager(value))
        else:
            form.ignoreRequest = False
            form.ignoreContent = True
        return form.widgetFactory.widget(field)

    def addValueWidget(self, new_identifier, value):
        widget = self.createValueWidget(new_identifier, value)
        if widget is not None:
            self.valueWidgets.append(widget)
        return widget

    def prepareContentValue(self, values):
        count = 0
        if values is not NO_VALUE:
            for position, value in enumerate(values):
                # Create new widgets for each value
                self.addValueWidget(position, value)
            count += len(values)
        if self.allowAdding and self.required and not count:
            self.addValueWidget(count, None)
            count += 1
        if count:
            self.haveValues = True
        return {self.identifier: str(count)}

    def prepareRequestValue(self, values, extractor):
        value_count = 0
        errors = None
        identifier_count = int(values.get(self.identifier, '0'))
        remove_something = self.identifier + '.remove' in values
        add_something = self.identifier + '.add' in values

        if self.inlineValidation:
            # If inlineValidation is on, and we removed or added
            # something, we extract this field to get the
            # validation messages right away (if the user clicked
            # on add or remove, he cannot have clicked on an
            # action button)
            if add_something or remove_something:
                ignored, errors = extractor.extract()
                if errors:
                    self.form.errors.append(errors)

        for position in range(0, identifier_count):
            value_marker = (self.identifier, position,)
            value_present = '%s.present.%d' % value_marker in values
            if not value_present:
                continue
            value_identifier = '%s.field.%d' % value_marker
            value_selected = '%s.checked.%d' % value_marker in values
            if remove_something and value_selected:
                if errors and value_identifier in errors:
                    # If the field have an error, remove it
                    del errors[value_identifier]
                continue
            # We need to provide the widget error now, but cannot set
            # all of them on the form now, as we might remove them
            # with delete
            self.addValueWidget(position, None)
            value_count += 1
        if (add_something or
            (self.allowAdding and self.required and not value_count)):
            self.addValueWidget(identifier_count, None)
            value_count += 1
            values[self.identifier] = str(identifier_count + 1)
        if value_count:
            self.haveValues = True
        if errors:
            if len(errors) > 1:
                self.form.errors.append(
                    Error(_(u"There were errors."), self.form.prefix))
            else:
                # If no errors  are left, remove from the form (top level error)
                del self.form.errors[self.identifier]
        return values

    @property
    def jsonTemplateWidget(self):
        widgets = Widgets()
        widgets.append(self.createValueWidget('{identifier}', None))
        widgets.update()
        return list(widgets)[0]

    def update(self):
        super(MultiGenericFieldWidget, self).update()
        self.valueWidgets.update()
        requireCollectionResources()

        self.jsonAddIdentifier = None
        self.jsonAddTemplate = None
        self.includeEmptyMessage = self.allowRemove
        if self.allowAdding:
            self.jsonAddIdentifier = 'id' + md5hash(self.identifier)
            widgets = Widgets()
            widgets.append(self.createValueWidget(
                    '{' + self.jsonAddIdentifier + '}', None))
            widgets.update()
            self.jsonAddTemplate = list(widgets)[0]


class ListGenericFieldWidget(MultiGenericFieldWidget):
    grok.adapts(ListField, Interface, Interface, Interface)

    def __init__(self, field, value_field, form, request):
        super(ListGenericFieldWidget, self).__init__(
            field, value_field, form, request)
        self.allowOrdering = field.allowOrdering


class MultiGenericDisplayFieldWidget(MultiGenericFieldWidget):
    grok.name('display')


# For collection of objects, generate a different widget (with a table)

class MultiObjectFieldWidget(MultiGenericFieldWidget):
    grok.adapts(ICollectionField, ObjectField, Interface, Interface)

    def getFields(self):
        return self.valueField.objectFields


class ListObjectFieldWidget(MultiObjectFieldWidget):
    grok.adapts(ListField, ObjectField, Interface, Interface)

    def __init__(self, field, value_field, form, request):
        super(ListObjectFieldWidget, self).__init__(
            field, value_field, form, request)
        self.allowOrdering = field.allowOrdering


# Still make possible to have the non-table implementation

class RegularMultiObjectFieldWidget(MultiGenericFieldWidget):
    grok.adapts(ICollectionField, ObjectField, Interface, Interface)
    grok.name('input-list')


class RegularListObjectFieldWidget(ListGenericFieldWidget):
    grok.adapts(ListField, ObjectField, Interface, Interface)
    grok.name('input-list')


class MultiGenericWidgetExtractor(WidgetExtractor):
    grok.adapts(ICollectionField, Interface, Interface, Interface)

    def __init__(self, field, value_field, form, request):
        super(MultiGenericWidgetExtractor, self).__init__(
            field, form, request)
        self.valueField = value_field

    def extract(self):
        value = self.request.form.get(self.identifier, NO_VALUE)
        if value is not NO_VALUE:
            try:
                value = int(value)
            except ValueError:
                return (None, u"Invalid internal input")
            collectedValues = []
            collectedErrors = Errors(identifier=self.identifier)
            for position in range(0, value):
                value_present = '%s.present.%d' % (
                    self.identifier, position) in self.request.form
                if not value_present:
                    # This value have been removed
                    continue
                field = self.valueField.clone(new_identifier=str(position))
                form = cloneFormData(self.form, prefix=self.identifier)
                data, errors = form.extractData(Fields(field))
                if errors:
                    collectedErrors.extend(errors)
                else:
                    collectedValues.append(data[field.identifier])
            if collectedErrors:
                return (None, collectedErrors)
            value = self.component.collectionType(collectedValues)
        return (value, None)


# Multi-Choice widget

class MultiChoiceFieldWidget(ChoiceFieldWidget):
    grok.adapts(SetField, ChoiceField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-multichoice']

    def __init__(self, field, value_field, form, request):
        super(MultiChoiceFieldWidget, self).__init__(field, form, request)
        self.source = value_field

    def prepareContentValue(self, value):
        form_value = []
        if value is NO_VALUE:
            return {self.identifier: form_value}
        choices = self.choices()
        for entry in value:
            try:
                term = choices.getTerm(entry)
                form_value.append(term.token)
            except LookupError:
                pass
        return {self.identifier: form_value}

    def renderableChoice(self):
        current = self.inputValue()
        base_id = self.htmlId()
        for i, choice in enumerate(self.choices()):
            yield {'token': choice.token,
                   'title': choice.title,
                   'checked': choice.token in current,
                   'id': base_id + '-' + str(i)}


grok.global_adapter(
    newCollectionWidgetFactory(mode='multiselect'),
    adapts=(ICollectionField, Interface, Interface),
    provides=IWidget,
    name='multiselect')


class MultiSelectFieldWidget(MultiChoiceFieldWidget):
    grok.name('multiselect')


class MultiChoiceDisplayFieldWidget(MultiChoiceFieldWidget):
    grok.name('display')

    def renderableChoice(self):
        current = self.inputValue()
        base_id = self.htmlId()
        for i, choice in enumerate(self.choices()):
            if choice.token in current:
                yield {'title': choice.title,
                       'id': base_id + '-' + str(i)}


class MultiChoiceWidgetExtractor(WidgetExtractor):
    grok.adapts(SetField, ChoiceField, Interface, Interface)

    def __init__(self, field, value_field, form, request):
        super(MultiChoiceWidgetExtractor, self).__init__(field, form, request)
        self.source = value_field

    def extract(self):
        value, errors = super(MultiChoiceWidgetExtractor, self).extract()
        if errors is None:
            is_present = self.request.form.get(
                self.identifier + '.present', NO_VALUE)
            if is_present is NO_VALUE:
                # Not in the request
                return (NO_VALUE, None)
            if value is NO_VALUE:
                # Nothing selected
                return (self.component.collectionType(), None)
            choices = self.source.getChoices(self.form)
            try:
                if not isinstance(value, list):
                    value = [value]
                value = self.component.collectionType(
                    [choices.getTermByToken(t).value for t in value])
            except LookupError:
                return (None, _(u'The selected value is not available.'))
        return (value, errors)


def makeCollectionSchemaFactory(factory):

    def CollectionSchemaFactory(schema):
        field = factory(
            schema.title or None,
            identifier=schema.__name__,
            description=schema.description,
            required=schema.required,
            readonly=schema.readonly,
            minLength=schema.min_length,
            maxLength=schema.max_length,
            valueField=schema.value_type,
            interface=schema.interface,
            constrainValue=schema.constraint,
            defaultFactory=schema.defaultFactory,
            defaultValue=schema.__dict__['default'] or NO_VALUE)
        return field

    return CollectionSchemaFactory


def register():
    registerSchemaField(
        makeCollectionSchemaFactory(CollectionField),
        schema_interfaces.ICollection)
    registerSchemaField(
        makeCollectionSchemaFactory(ListField),
        schema_interfaces.IList)
    registerSchemaField(
        makeCollectionSchemaFactory(SetField),
        schema_interfaces.ISet)
    registerSchemaField(
        makeCollectionSchemaFactory(TupleField),
        schema_interfaces.ITuple)



# -*- coding: utf-8 -*-

import zeam.form.ztk
from grokcore.component import zcml
from zope.app.wsgi.testlayer import BrowserLayer
from zope.configuration.config import ConfigurationMachine


FunctionalLayer = BrowserLayer(zeam.form.ztk)

def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok(module_name, config)
    config.execute_actions()



# -*- coding: utf-8 -*-

from zeam.form.base.markers import Marker, NO_VALUE
from zeam.form.base.widgets import FieldWidget
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18nmessageid import MessageFactory
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


class PasswordField(Field):
    """A password field.
    """

    def __init__(self, title,
                 minLength=0,
                 maxLength=None,
                 **options):
        super(PasswordField, self).__init__(title, **options)
        self.minLength = minLength
        self.maxLength = maxLength

    def isEmpty(self, value):
        return value is NO_VALUE or not len(value)

    def validate(self, value, form):
        error = super(PasswordField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker) and len(value):
            assert isinstance(value, basestring)
            if self.minLength and len(value) < self.minLength:
                return _(u"This password is too short.")
            if self.maxLength and len(value) > self.maxLength:
                return _(u"This password is too long.")
        return None


class PasswordWidget(FieldWidget):
    grok.adapts(PasswordField, None, None)
    defaultHtmlClass = ['field', 'field-password']
    defaultHtmlAttributes = ['readonly', 'required', 'autocomplete',
                             'maxlength', 'pattern', 'placeholder',
                             'size', 'style']


def PasswordSchemaFactory(schema):
    field = PasswordField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        minLength=schema.min_length,
        maxLength=schema.max_length,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def register():
    registerSchemaField(PasswordSchemaFactory, schema_interfaces.IPassword)




from zeam.form import base
from zope import interface, schema

from grokcore import component as grok


class IPerson(interface.Interface):
    name = schema.TextLine(
        title=u"Person name")
    age = schema.Int(
        title=u"Person age",
        description=u"Age in years")


class Person(grok.Context):
    grok.implements(IPerson)


class PersonForm(base.Form):

    label = u"People form"
    description = u"Form to send people outerspace"

    fields = base.Fields(IPerson)

    @base.action(u"Send")
    def send(self):
        data, errors = self.extractData()
        self.status = u"We sent %s, age %d" % (data['name'], data['age'])




# -*- coding: utf-8 -*-

from zeam.form.base.interfaces import IFieldExtractionValueSetting
from zeam.form.base.markers import Marker, NO_VALUE
from zeam.form.base.widgets import FieldWidget, FieldWidgetExtractor
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


class IntegerField(Field):
    """A integer field.
    """

    def __init__(self, title,
                 min=None,
                 max=None,
                 **options):
        # We pass min and max to Field to have them in htmlAttributes as well
        super(IntegerField, self).__init__(title, min=min, max=max, **options)
        self.min = min
        self.max = max

    def validate(self, value, form):
        error = super(IntegerField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker):
            assert isinstance(value, (int, long, float))
            if self.min is not None and value < self.min:
                return _(u"This number is too small.")
            if self.max is not None and value > self.max:
                return _(u"This number is too big.")
        return None

# BBB
IntSchemaField = IntegerField


class IntegerFieldWidgetExtractor(FieldWidgetExtractor):
    grok.adapts(IntegerField, IFieldExtractionValueSetting, Interface)
    valueType = int
    failedMessage = _(u"This number is not a valid whole number.")

    def extract(self):
        value, error = super(IntegerFieldWidgetExtractor, self).extract()
        if error:
            return (value, error)
        if value is not NO_VALUE and len(value):
            try:
                value = self.valueType(value)
            except (ValueError, TypeError):
                return (NO_VALUE, self.failedMessage)
            return (value, None)
        return (NO_VALUE, None)


class FloatField(IntegerField):
    """A float field.
    """


# BBB
FloatSchemaField = FloatField


class CurrencyField(FloatField):
    """ A currency field.
    """

    def __init__(self, min=None,
                       max=None,
                       symbol=u'€',
                       thousandsSeparator=u',',
                       decimalSeparator=u'.',
                       symbolPrecedes=False,
                       symbolSpaceSepartor=True,
                       fracDigits=2,
                       positiveSign=u'',
                       negativeSign=u'-',
                       signPrecedes=True,
                       **options):
        super(CurrencyField, self).__init__(max=max, min=min, **options)
        self.symbol = symbol
        self.thousandsSeparator = thousandsSeparator
        self.decimalSeparator = decimalSeparator
        self.symbolPrecedes = symbolPrecedes
        self.symbolSpaceSepartor = symbolSpaceSepartor
        self.fracDigits = fracDigits
        self.positiveSign = positiveSign
        self.negativeSign = negativeSign
        self.signPrecedes = signPrecedes


class FloatFieldWidgetExtractor(IntegerFieldWidgetExtractor):
    grok.adapts(FloatField, IFieldExtractionValueSetting, Interface)
    valueType = float
    failedMessage = _(u"This number is not a valid decimal number.")


class NumberWidget(FieldWidget):
    grok.adapts(IntegerField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-number']
    defaultHtmlAttributes = set(['readonly', 'required', 'autocomplete',
                                 'max', 'min', 'setup', 'placeholder',
                                 'style'])


class CurrencyDisplayWidget(FieldWidget):
    grok.adapts(CurrencyField, Interface, Interface)
    grok.name('display')

    defaultHtmlClass = ['field', 'field-currency']

    def valueToUnicode(self, value):
        return self.formatHtmlCurrency(value)

    def formatHtmlCurrency(self, value):
        string_value = ("%%.0%df" % self.component.fracDigits) % abs(value)
        integer_part, decimal_part = string_value.split('.')
        digits = list(integer_part)
        chars = []
        count = 0
        while digits:
            digit = digits.pop()
            chars.append(digit)
            count += 1
            if count % 3 == 0 and len(digits):
                chars.append(self.component.thousandsSeparator)
        integer_part = "".join(reversed(chars))
        buf = u''
        if self.component.symbolPrecedes:
            buf += self.component.symbol
            if self.component.symbolSpaceSepartor:
                buf += '&nbsp;'
        if value >= 0:
            buf += self.component.positiveSign
        else:
            buf += self.component.negativeSign
        buf += integer_part + self.component.decimalSeparator + decimal_part
        if not self.component.symbolPrecedes:
            if self.component.symbolSpaceSepartor:
                buf += '&nbsp;'
            buf += self.component.symbol
        return buf


def IntegerSchemaFactory(schema):
    field = IntegerField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        min=schema.min,
        max=schema.max,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def FloatSchemaFactory(schema):
    field = FloatField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        min=schema.min,
        max=schema.max,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def register():
    registerSchemaField(FloatSchemaFactory, schema_interfaces.IFloat)
    registerSchemaField(IntegerSchemaFactory, schema_interfaces.IInt)



# -*- coding: utf-8 -*-

from zope import interface
from zope.schema.interfaces import ISource
from zeam.form.base.interfaces import IField, IZeamFormBaseAPI


class IFormSourceBinder(ISource):
    """Marker interface used with zope.schema Choice in order to give
    a factory that takes the source.

    """

class ISchemaField(IField):
    """This is a field for zope schema field.
    """

    def fromUnicode(value):
        """This convert a value from a unicode string (or not).
        """


class IFieldCreatedEvent(interface.Interface):
    """A zeam.form Field have been created (out of a zope.schema
    Field).
    """
    field = interface.Attribute(
        u"Created field")
    interface = interface.Attribute(
        u"Interface on which the field have been crated")


class ICollectionField(IField):
    """This is a field for zope schema collection field. It is defined
    in order to be able to have a generic behavior on collections.
    """
    collectionType = interface.Attribute(
        u"Python type represented by this collection (like set, list...)")
    valueField = interface.Attribute(
        u"Field corresponding to the value type contained in the collection")
    allowAdding = interface.Attribute(
        u"Boolean flag to allow adding of values to the collection")
    allowRemove = interface.Attribute(
        u"Boolean flag to allow removing of values from the collection")
    inlineValidation = interface.Attribute(
        u"Boolean flag to validate data when a value is added or removed")


class IListField(ICollectionField):
    """This is a field for zope schema list field.
    """
    allowOrdering = interface.Attribute(
        u"Boolean flag to allow ordering of values")


class IObjectField(IField):
    """This field is mapped to the zope.schema Object schema field.
    """
    objectFactory = interface.Attribute(
        u"Default factory used to create new objects for the field")
    objectSchema = interface.Attribute(
        u"Zope Interface that should be implemented by the object")
    objectFields = interface.Attribute(
        u"Zeam fields for the object")

    def getObjectFactory():
        """Return the object factory used to create new objects for
        the field.
        """


class IZeamFormZTKAPI(IZeamFormBaseAPI):
    """API exported by zeam.form.ztk.
    """
    customize = interface.Attribute(
        u"Decorator used to customize created fields")

    AddAction = interface.Attribute(
        u"Action to add a new content to one other")
    EditAction = interface.Attribute(
        u"Action to edit the values of a content")
    CancelAction = interface.Attribute(
        u"Action to cancel the submission of the form")



# -*- coding: utf-8 -*-

from zope.interface.interfaces import IMethod
from zope.interface import directlyProvides, Invalid
from zeam.form.base.markers import NO_VALUE
from zeam.form.ztk.interfaces import ISchemaField


class Data(object):
    """Wraps the data dicts into an object, acting as
    an attribute access proxy.
    """
    def __init__(self, interface, data):
        self.interface = interface
        self.data = data
        directlyProvides(self, interface)

    def __getattr__(self, name):
        try:
            field = self.interface[name]
        except KeyError:
            raise AttributeError(name)

        if IMethod.providedBy(field):
            raise RuntimeError("Data value is not a schema field", name)

        value = self.data.getWithDefault(name)

        if value is NO_VALUE:
            raise AttributeError(name)

        return value


class InvariantsValidation(object):
    """Validates the invariants of the given fields' interfaces.
    """
    def __init__(self, form, fields):
        self.form = form
        self.interfaces = []
        for field in fields:
            if (field.interface is not None and
                field.interface not in self.interfaces):
                self.interfaces.append(field.interface)

    def validate(self, data):
        errors = []
        for interface in self.interfaces:
            obj = Data(interface, data)
            try:
                interface.validateInvariants(obj, errors)
            except Invalid, error:
                pass
            for error in errors:
                yield error.args[0]




from zeam.form import ztk
from zope import interface, schema
from zope.schema.fieldproperty import FieldProperty

from grokcore import component as grok


class IComment(interface.Interface):
    title = schema.TextLine(
        title=u"Title",
        required=True)
    comment = schema.Text(
        title=u"Comment",
        required=True)
    name = schema.TextLine(
        title=u"Name",
        default=u'',
        required=False)


class Comment(grok.Context):
    grok.implements(IComment)

    title = FieldProperty(IComment['title'])
    comment = FieldProperty(IComment['comment'])
    name = FieldProperty(IComment['name'])

    def __init__(self, title, comment, name=u''):
        self.title = title
        self.comment = comment
        self.name = name


class Edit(ztk.Form):
    label = u"Modify your comment"

    ignoreContent = False
    fields = ztk.Fields(IComment)
    actions = ztk.Actions(ztk.EditAction(u"Change"))


from zope.security.protectclass import protectName, protectSetAttribute
# Need to declare security for Zope madness

protectName(Comment, 'title', 'zope.Public')
protectName(Comment, 'comment', 'zope.Public')
protectName(Comment, 'name', 'zope.Public')

# Everybody has edit right, so test are simpler
protectSetAttribute(Comment, 'title', 'zope.Public')
protectSetAttribute(Comment, 'comment', 'zope.Public')
protectSetAttribute(Comment, 'name', 'zope.Public')



# -*- coding: utf-8 -*-

from zeam.form.base.widgets import FieldWidget, DisplayFieldWidget
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18nmessageid import MessageFactory
from zope.schema import interfaces as schema_interfaces


_ = MessageFactory("zeam.form.base")


class BooleanField(Field):
    """A boolean field.
    """

# BBB
BooleanSchemaField = BooleanField


class CheckBoxWidget(FieldWidget):
    grok.adapts(BooleanField, None, None)
    defaultHtmlClass = ['field', 'field-bool']
    defaultHtmlAttributes = set(['readonly', 'style', 'disabled'])
    alternateLayout = True


class CheckBoxDisplayWidget(DisplayFieldWidget):
    grok.adapts(BooleanField, None, None)

    def valueToUnicode(self, value):
        if bool(value):
            return _(u'Yes')
        return _(u'No')


class CheckBoxWidgetExtractor(WidgetExtractor):
    grok.adapts(BooleanField, None, None)

    def extract(self):
        value, error = WidgetExtractor.extract(self)
        if value == 'True':
            value = True
        else:
            value = False
        return (value, error)


def BooleanSchemaFactory(schema):
    field = BooleanField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=bool(schema.__dict__['default']))
    return field


def register():
    registerSchemaField(BooleanSchemaFactory, schema_interfaces.IBool)



# -*- coding: utf-8 -*-

from zeam.form.base import interfaces
from zeam.form.base.fields import Field as BaseField
from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import FieldWidget, WidgetExtractor
from zeam.form.ztk.interfaces import ISchemaField, IFieldCreatedEvent

from grokcore import component as grok
from zope import schema, component
from zope.event import notify
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface, Invalid
from zope.schema import interfaces as schema_interfaces
from zope.schema._bootstrapinterfaces import IContextAwareDefaultFactory
import zope.interface.interfaces

_ = MessageFactory("zeam.form.base")


class FieldCreatedEvent(object):
    grok.implements(IFieldCreatedEvent)

    def __init__(self, field, interface=None, origin=None):
        self.interface = interface
        self.field = field
        self.origin = origin


class SchemaFieldFactory(object):
    """Create form fields from a zope.schema field (by adapting it).
    """
    grok.implements(interfaces.IFieldFactory)

    def __init__(self, context):
        self.context = context

    def produce(self):
        interface = self.context.interface
        if interface is None and not getattr(self.context, '__name__', None):
            raise AssertionError("Field has no interface or __name__")
        result = interfaces.IField(self.context)
        notify(FieldCreatedEvent(result, interface, self.context))
        yield result


class InterfaceSchemaFieldFactory(object):
    """Create a set of form fields from a zope.interface by looking
    each zope.schema fields defined on it and adapting them.
    """
    grok.implements(interfaces.IFieldFactory)

    def __init__(self, context):
        self.context = context

    def produce(self):
        for name, field in schema.getFieldsInOrder(self.context):
            result =  interfaces.IField(field)
            notify(FieldCreatedEvent(result, self.context, field))
            yield result


class Field(BaseField):

    defaultFactory = None
    
    def __init__(self, *args, **kwargs):
        if 'defaultFactory' in kwargs:
             self.defaultFactory = kwargs.pop('defaultFactory')
        super(Field, self).__init__(*args, **kwargs)

    def getDefaultValue(self, form):
        if self.defaultFactory is not None:
            if IContextAwareDefaultFactory.providedBy(self.defaultFactory):
                if form is None:
                    raise TypeError('defaultFactory context required.')
                default = self.defaultFactory(form.getContent()) 
            else: 
                default = self.defaultFactory()
        else:
            default = super(Field, self).getDefaultValue(form)

        if default is NO_VALUE:
            default = self.defaultValue

        if default is None:
            return NO_VALUE

        return default
        
            
class SchemaField(BaseField):
    """A form field using a zope.schema field as settings.
    """
    grok.implements(ISchemaField)

    def __init__(self, field):
        super(SchemaField, self).__init__(
            field.title or None, field.__name__,
            description=field.description,
            required=field.required,
            readonly=field.readonly,
            interface=field.interface)
        self._field = field

    def get_field(self):
        return self._field

    def clone(self, new_identifier=None):
        copy = self.__class__(self._field)
        copy.__dict__.update(self.__dict__)
        if new_identifier is not None:
            copy.identifier = new_identifier
        return copy

    def validate(self, value, form):
        error = super(SchemaField, self).validate(value, form)
        if error is not None:
            return error

        if value is not NO_VALUE:
            context = None
            if form is not None:
                context = form.context
            try:
                binded_field = self._field.bind(context)
                binded_field.validate(value)
            except schema_interfaces.ValidationError, error:
                return error.doc()
            except Invalid, error:
                return error.args[0]
        return None

    def fromUnicode(self, value):
        if schema_interfaces.IFromUnicode.providedBy(self._field):
            return self._field.fromUnicode(value)
        return value

    def getDefaultValue(self, form):
        default = super(SchemaField, self).getDefaultValue(form)
        if default is not NO_VALUE:
            return default
        default = self._field.default
        if default is None:     # Zope schema use None to say no default
            return NO_VALUE
        return default


def registerSchemaField(factory, schema_field):
    # We register it by hand to have the adapter available when loading ZCML.
    component.provideAdapter(factory, (schema_field,), interfaces.IField)


class SchemaFieldWidget(FieldWidget):
    grok.adapts(ISchemaField, Interface, Interface)

    def htmlClass(self):
        css_class = ['field']
        if ISchemaField.providedBy(self.component):
            # Prevent old FieldWidget to fail if they haven't been
            # updated to the new API.
            css_class.append('field-%s' % (
                    self.component._field.__class__.__name__.lower()))
        if self.required:
            css_class.append('field-required')
        return ' '.join(css_class)


class SchemaWidgetExtractor(WidgetExtractor):
    grok.adapts(ISchemaField, Interface, Interface)

    def extract(self):
        value, error = super(SchemaWidgetExtractor, self).extract()
        if error is not None:
            return value, error

        if value is not NO_VALUE:
            try:
                value = self.component.fromUnicode(value)
            except schema_interfaces.ValidationError, e:
                return None, e.doc()
            except Invalid, e:
                return None, e.args[0]
            except ValueError, e:
                return None, _(u"Invalid value.")

        return value, None


class HiddenSchemaWidgetExtractor(SchemaWidgetExtractor):
    grok.name('hidden')


class ReadOnlySchemaWidgetExtractor(SchemaWidgetExtractor):
    grok.name('readonly')


def registerDefault():
    """Register default fields factories.
    """
    component.provideAdapter(
        SchemaFieldFactory,
        (zope.schema.interfaces.IField,))
    component.provideAdapter(
        InterfaceSchemaFieldFactory,
        (zope.interface.interfaces.IInterface,))
    registerSchemaField(SchemaField, schema_interfaces.IField)




import unittest

from zope.testing import doctest

from zeam.form.ztk.testing import FunctionalLayer

def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    globs= {}

    suite = unittest.TestSuite()
    for filename in ['fields.txt', 'validation.txt']:
        test = doctest.DocFileSuite(
            filename,
            optionflags=optionflags,
            globs=globs)
        test.layer = FunctionalLayer
        suite.addTest(test)

    return suite



# -*- coding: utf-8 -*-
"""
We are going to use a simple form with an edit action to edit a comment.

Let's grok our example:

  >>> from zeam.form.ztk.testing import grok
  >>> grok('zeam.form.ztk.ftests.forms.editform_fixture')

Let's add a comment and try to edit it with our form:

  >>> from zeam.form.ztk.ftests.forms.editform_fixture import Comment
  >>> root = getRootFolder()
  >>> root['comment'] = Comment(u'zeam.form', u'Is great')
  >>> root['comment'].title
  u'zeam.form'
  >>> root['comment'].comment
  u'Is great'
  >>> root['comment'].name
  u''

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

Now acccess the edit form:

  >>> browser.open('http://localhost/comment/edit')
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')

  >>> u'Modify your comment' in browser.contents
  True

  >>> title_field = browser.getControl('Title')
  >>> title_field
  <Control name='form.field.title' type='text'>
  >>> title_field.value
  'zeam.form'

  >>> comment_field = browser.getControl('Comment')
  >>> comment_field
  <Control name='form.field.comment' type='textarea'>
  >>> comment_field.value
  'Is great'

  >>> name_field = browser.getControl('Name')
  >>> name_field
  <Control name='form.field.name' type='text'>
  >>> name_field.value
  ''

We can now edit the content, and so it get modified:

  >>> title_field.value = 'zeam.form.ztk'
  >>> comment_field.value = 'Is far cooler than not ztk'
  >>> name_field.value = 'Arthur de la contee d`or'
  >>> change_button = browser.getControl('Change')
  >>> change_button
  <SubmitControl name='form.action.change' type='submit'>

  >>> change_button.click()
  >>> 'Modification saved' in browser.contents
  True

Modifications are saved, we have the new value if we reload the page:

  >>> browser.open('http://localhost/comment/edit')

  >>> title_field = browser.getControl('Title')
  >>> title_field.value
  'zeam.form.ztk'
  >>> comment_field = browser.getControl('Comment')
  >>> comment_field.value
  'Is far cooler than not ztk'
  >>> name_field = browser.getControl('Name')
  >>> name_field.value
  'Arthur de la contee d`or'

We can remove name, it will work as it is not required:

  >>> name_field.value = ''
  >>> change_button = browser.getControl('Change')
  >>> change_button.click()
  >>> 'Modification saved' in browser.contents
  True


And name is gone:

  >>> browser.open('http://localhost/comment/edit')

  >>> title_field = browser.getControl('Title')
  >>> title_field.value
  'zeam.form.ztk'
  >>> comment_field = browser.getControl('Comment')
  >>> comment_field.value
  'Is far cooler than not ztk'
  >>> name_field = browser.getControl('Name')
  >>> name_field.value
  ''

However comment is required:

  >>> comment_field.value = ''
  >>> change_button = browser.getControl('Change')
  >>> change_button.click()
  >>> 'Modification saved' in browser.contents
  False
  >>> 'There were errors' in browser.contents
  True

So no changes happened:

  >>> browser.open('http://localhost/comment/edit')

  >>> title_field = browser.getControl('Title')
  >>> title_field.value
  'zeam.form.ztk'
  >>> comment_field = browser.getControl('Comment')
  >>> comment_field.value
  'Is far cooler than not ztk'
  >>> name_field = browser.getControl('Name')
  >>> name_field.value
  ''

"""




import unittest
from zope.testing import doctest
from zeam.form.ztk.testing import FunctionalLayer

def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    globs= {}

    suite = unittest.TestSuite()
    for filename in ['bool.txt', 'choice.txt', 'collection_set.txt',
                     'collection_list.txt', 'collection_object.txt',
                     'multichoice.txt', 'object.txt', 'date.txt',
                     'radio.txt', 'uri.txt', 'text.txt', 'time.txt',
                     'currency.txt', 'number.txt']:
        test = doctest.DocFileSuite(
            filename,
            optionflags=optionflags,
            globs=globs)
        test.layer = FunctionalLayer
        suite.addTest(test)

    return suite




import sys
import martian
from martian.util import frame_is_module
from martian.error import GrokImportError

from zope.event import subscribers
from zope.interface.adapter import AdapterRegistry
from zope.interface import providedBy, implementedBy, Interface

from zeam.form.ztk.interfaces import IFieldCreatedEvent


class CustomizationRegistry(object):
    """Register and execute customization handlers.
    """

    def __init__(self):
        self._scheduled = False
        self._events = []
        self._origin = AdapterRegistry()
        self._field = AdapterRegistry()
        subscribers.append(self.watch)

    def watch(self, *events):
        if len(events) > 0 and IFieldCreatedEvent.providedBy(events[0]):
            self._events.append(events[0])

    def register(self, handler, options):
        if 'origin' in options:
            self._origin.register(
                (options['origin'], options.get('schema')),
                Interface,
                options.get('name', u''),
                handler)
        elif 'field' in options:
            self._field.register(
                (options['field'], options.get('schema')),
                Interface,
                options.get('name', u''),
                handler)
        elif 'schema' in options:
            self._origin.register(
                (Interface, options['schema']),
                Interface,
                options.get('name', u''),
                handler)
        else:
            raise AssertionError('Invalid customization')

    def execute(self, clear=True):
        for event in self._events:
            handler = None
            if event.origin is not None:
                # 1. Lookup customization with the original field
                required = (providedBy(event.origin), event.interface)
                # 1.a Original field and name
                handler = self._origin.lookup(
                    required, Interface, event.field.identifier)
                if handler is None:
                    # 1.b Original field without name
                    handler = self._origin.lookup(required, Interface)
                if handler is not None:
                    handler(event.field)
            if handler is None:
                # 2. No customization found, lookup with the zeam.form field
                required = (providedBy(event.field), event.interface)
                # 2.a zeam.form field and name
                handler = self._field.lookup(
                    required, Interface, event.field.identifier)
                if handler is None:
                    # 2.b zeam.form field without name
                    handler = self._field.lookup(required, Interface)
                if handler is not None:
                    handler(event.field)
        if clear:
            del self._events[:]
        self._scheduled = False

    def schedule(self, config):
        if not self._scheduled:
            config.action(
                discriminator=('customize fields',),
                callable=self.execute,
                args=tuple(),
                order=sys.maxint-1)
            self._scheduled = True

registry = CustomizationRegistry()


class customize:

    def __init__(self, origin=None, schema=None, field=None, name=u''):
        self.options = {'origin': origin, 'schema': schema,
                        'field':field, 'name': name}

    def __call__(self, handler):
        frame = sys._getframe(1)
        if not frame_is_module(frame):
            raise GrokImportError(
                '@customize can only be used on module level')
        if not self.options:
            raise GrokImportError(
                '@customize options are missing')

        customizations = frame.f_locals.get(
            '__zeam_form_customizations__', None)
        if customizations is None:
            frame.f_locals['__zeam_form_customizations__'] = customizations = []
        customizations.append((handler, self.options))
        return handler


class CustomizationGrokker(martian.GlobalGrokker):

    def grok(self, name, module, module_info, config, *kw):
        customizations = module_info.getAnnotation(
            'zeam.form.customizations', [])
        if customizations:
            for customization in customizations:
                registry.register(customization[0], customization[1])
            registry.schedule(config)
            return True
        return False



# -*- coding: utf-8 -*-

from zeam.form.base.datamanager import ObjectDataManager
from zeam.form.base.errors import Errors
from zeam.form.base.fields import Fields
from zeam.form.base.form import cloneFormData
from zeam.form.base.markers import NO_VALUE, Marker, DEFAULT
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.base.widgets import Widgets, FieldWidget
from zeam.form.ztk.fields import Field, registerSchemaField
from zeam.form.ztk.interfaces import IObjectField

from grokcore import component as grok
from zope.component import getUtility
from zope.component.interfaces import IFactory
from zope.interface import Interface, implements
from zope.schema import interfaces as schema_interfaces


class ObjectField(Field):
    """A collection field.
    """
    implements(IObjectField)
    objectFactory = DEFAULT
    dataManager = ObjectDataManager

    def __init__(self, title, schema=None, **options):
        super(ObjectField, self).__init__(title, **options)
        self._schema = schema
        self._fields = Fields()
        if schema is not None:
            self._fields.extend(schema)

    @property
    def objectSchema(self):
        return self._schema

    @property
    def objectFields(self):
        return self._fields

    def getObjectFactory(self):
        if self.objectFactory is not DEFAULT:
            return self.objectFactory
        schema = self.objectSchema
        return getUtility(IFactory, name=schema.__identifier__)

# BBB
ObjectSchemaField = ObjectField


class ObjectFieldWidget(FieldWidget):
    grok.adapts(ObjectField, Interface, Interface)

    def prepareContentValue(self, value):
        if value is NO_VALUE:
            return {self.identifier: []}
        return {self.identifier: value}

    def update(self):
        super(ObjectFieldWidget, self).update()
        value = self.component.dataManager(self.inputValue())
        form = cloneFormData(self.form, value, self.identifier)
        self.objectWidgets = Widgets(form=form, request=self.request)
        self.objectWidgets.extend(self.component.objectFields)
        self.objectWidgets.update()


class ObjectDisplayWidget(ObjectFieldWidget):
    grok.name('display')


class ObjectFieldExtractor(WidgetExtractor):
    grok.adapts(ObjectField, Interface, Interface)

    def extract(self):
        is_present = self.request.form.get(self.identifier, NO_VALUE)
        if is_present is NO_VALUE:
            return (NO_VALUE, None)
        value = None
        form = cloneFormData(self.form, None, self.identifier)
        data, errors = form.extractData(self.component.objectFields)
        if not errors:
            factory = self.component.getObjectFactory()
            # Create an object with values
            value = factory(**dict(filter(
                        lambda (k, v): not isinstance(v, Marker),
                        data.items())))
            return (value, None)
        return (value, Errors(*errors, identifier=self.identifier))


def ObjectSchemaFactory(schema):
    field = ObjectField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        schema=schema.schema,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def register():
    registerSchemaField(ObjectSchemaFactory, schema_interfaces.IObject)



# -*- coding: utf-8 -*-

from zeam.form.base.markers import Marker, NO_VALUE
from zeam.form.base.widgets import FieldWidget, DisplayFieldWidget
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18n.format import DateTimeParseError
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


class DateField(Field):
    """A date field.
    """
    valueLength = 'short'

    @property
    def valueType(self):        # Read-only
        return 'date'

    def __init__(self, title,
                 min=None,
                 max=None,
                 **options):
        super(DateField, self).__init__(title, **options)
        self.min = min
        self.max = max

    def getFormatter(self, form):
        return form.request.locale.dates.getFormatter(
            self.valueType, self.valueLength)

    def validate(self, value, form):
        error = super(DateField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker):
            if self.min is not None and value < self.min:
                formatter = self.getFormatter(form)
                return _(u"This date is before the ${not_before}.",
                         dict(not_before=formatter.format(self.min)))
            if self.max is not None and value > self.max:
                formatter = self.getFormatter(form)
                return _(u"This date is after the ${not_after}.",
                         dict(not_after=formatter.format(self.max)))
        return None

# BBB
DateSchemaField = DateField

class DatetimeField(DateField):
    """A datetime field.
    """
    valueLength = 'short'

    @property
    def valueType(self):        # Read-only
        return 'dateTime'

# BBB
DatetimeSchemaField = DatetimeField


class DateFieldWidget(FieldWidget):
    grok.adapts(DateField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-date']

    def valueToUnicode(self, value):
        formatter = self.component.getFormatter(self.form)
        return formatter.format(value)


class DateWidgetExtractor(WidgetExtractor):
    grok.adapts(DateField, Interface, Interface)

    def extract(self):
        value, error = super(DateWidgetExtractor, self).extract()
        if value is not NO_VALUE:
            if not len(value):
                return NO_VALUE, None
            formatter = self.component.getFormatter(self.form)
            try:
                value = formatter.parse(value)
            except (ValueError, DateTimeParseError), error:
                return None, str(error)
        return value, error


class DateFieldDisplayWidget(DisplayFieldWidget):
    grok.adapts(DateField, Interface, Interface)

    def valueToUnicode(self, value):
        formatter = self.component.getFormatter(self.form)
        return formatter.format(value)


def DateSchemaFactory(schema):
    field = DateField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        min=schema.min,
        max=schema.max,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field

def DatetimeSchemaFactory(schema):
    field = DatetimeField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        min=schema.min,
        max=schema.max,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field


def register():
    registerSchemaField(DatetimeSchemaFactory, schema_interfaces.IDatetime)
    registerSchemaField(DateSchemaFactory, schema_interfaces.IDate)



# -*- coding: utf-8 -*-

from zeam.form.base.markers import NO_VALUE, Marker
from zeam.form.base.widgets import FieldWidget
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import Field, registerSchemaField
from zeam.form.ztk.interfaces import IFormSourceBinder

from grokcore import component as grok
from zope import component
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.interfaces import IVocabularyTokenized, IVocabularyFactory

_ = MessageFactory("zeam.form.base")


class ChoiceField(Field):
    """A choice field.
    """
    _source = None
    _vocabularyFactory = None
    _vocabularyName = None

    def __init__(self, title,
                 source=None,
                 vocabularyName=None,
                 **options):
        super(ChoiceField, self).__init__(title, **options)
        if source is not None:
            self.source = source
        elif vocabularyName is not None:
            self.vocabularyFactory = vocabularyName

    @apply
    def vocabularyFactory():

        def getter(self):
            if self._vocabularyFactory is None:
                if self._vocabularyName is not None:
                    self._vocabularyFactory = component.getUtility(
                        schema_interfaces.IVocabularyFactory,
                        name=self._vocabularyName)
            return self._vocabularyFactory

        def setter(self, factory):
            if isinstance(factory, str):
                self._vocabularyName = factory
                self._vocabularyFactory = None
            else:
                self._vocabularyName = None
                self._vocabularyFactory = factory
            self._source = None

        return property(getter, setter)

    @apply
    def source():

        def getter(self):
            return self._source

        def setter(self, source):
            # Verify if this is a source or a vocabulary
            if IVocabularyTokenized.providedBy(source):
                self._source = source
            else:
                # Be sure to reset the source
                self._source = None
                self._vocabularyFactory = source

        return property(getter, setter)

    def getChoices(self, form):
        source = self.source
        if source is None:
            factory = self.vocabularyFactory
            assert factory is not None, \
                "No vocabulary source available."
            if (IContextSourceBinder.providedBy(factory) or
                IVocabularyFactory.providedBy(factory)):
                source = factory(form.context)
            elif IFormSourceBinder.providedBy(factory):
                source = factory(form)
            assert IVocabularyTokenized.providedBy(source), \
                "No valid vocabulary available, %s is not valid for %s" % (
                source, self)
        return source

    def validate(self, value, form):
        error = super(ChoiceField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker):
            choices = self.getChoices(form)
            if value not in choices:
                return _(u"The selected value is not among the possible choices.")
        return None

# BBB
ChoiceSchemaField = ChoiceField


class ChoiceFieldWidget(FieldWidget):
    grok.adapts(ChoiceField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-choice']
    defaultHtmlAttributes = set(['required', 'size', 'style', 'disabled'])
    _choices = None

    def __init__(self, field, form, request):
        super(ChoiceFieldWidget, self).__init__(field, form, request)
        self.source = field

    def lookupTerm(self, value):
        choices = self.choices()
        try:
            return choices.getTerm(value)
        except LookupError:
            # the stored value is invalid. fallback on the default one.
            default = self.component.getDefaultValue(self.form)
            if default is not NO_VALUE:
                return choices.getTerm(default)
        return None

    def valueToUnicode(self, value):
        term = self.lookupTerm(value)
        if term is not None:
            return term.token
        return u''

    def choices(self):
        if self._choices is not None:
            return self._choices
        # self.source is used instead of self.component in order to be
        # able to override it in subclasses.
        self._choices = self.source.getChoices(self.form)
        return self._choices


class ChoiceDisplayWidget(ChoiceFieldWidget):
    grok.name('display')

    def valueToUnicode(self, value):
        term = self.lookupTerm(value)
        if term is not None:
            return term.title
        return u''


class ChoiceWidgetExtractor(WidgetExtractor):
    grok.adapts(ChoiceField, Interface, Interface)

    def extract(self):
        value, error = super(ChoiceWidgetExtractor, self).extract()
        if value is not NO_VALUE:
            choices = self.component.getChoices(self.form)
            try:
                value = choices.getTermByToken(value).value
            except LookupError:
                return (None, u'Invalid value')
        return (value, error)


# Radio Widget

class RadioFieldWidget(ChoiceFieldWidget):
    grok.adapts(ChoiceField, Interface, Interface)
    grok.name('radio')

    def renderableChoices(self):
        current = self.inputValue()
        base_id = self.htmlId()
        for i, choice in enumerate(self.choices()):
            yield {'token': choice.token,
                   'title': choice.title or choice.token,
                   'checked': choice.token == current and 'checked' or None,
                   'id': base_id + '-' + str(i)}


def ChoiceSchemaFactory(schema):
    field = ChoiceField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        source=schema.vocabulary,
        vocabularyName=schema.vocabularyName,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field

def register():
    registerSchemaField(ChoiceSchemaFactory, schema_interfaces.IChoice)



# this is a package.

from zeam.form.ztk.widgets.bool import BooleanField
from zeam.form.ztk.widgets.choice import ChoiceField
from zeam.form.ztk.widgets.date import DateField, DatetimeField
from zeam.form.ztk.widgets.number import IntegerField, FloatField, CurrencyField
from zeam.form.ztk.widgets.object import ObjectField
from zeam.form.ztk.widgets.password import PasswordField
from zeam.form.ztk.widgets.text import TextField
from zeam.form.ztk.widgets.textline import TextLineField
from zeam.form.ztk.widgets.time import TimeField
from zeam.form.ztk.widgets.uri import URIField


__all__ = ["BooleanField", "ChoiceField", "DateField", "DatetimeField",
           "IntegerField", "FloatField", "TextField", "PasswordField",
           "TextLineField", "TimeField", "URIField", "ObjectField",
           "CurrencyField"]




from zeam.form.base import *
from zeam.form.ztk.actions import AddAction, EditAction, CancelAction
from zeam.form.ztk.validation import InvariantsValidation
from zeam.form.ztk.interfaces import IZeamFormZTKAPI
from zeam.form.ztk.customizations import customize

__all__ = list(IZeamFormZTKAPI)



from setuptools import setup, find_packages
import os

version = '1.3.5'

tests_require = [
    'zope.app.schema',
    'zope.app.wsgi',
    'zope.security',
    'zope.testing',
    'zeam.form.base [test]'
    ]

setup(name='zeam.form.ztk',
      version=version,
      description="Zope Toolkit support for zeam.form",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Programming Language :: Zope",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zeam form zope schema edit content',
      author='Sylvain Viollon',
      author_email='thefunny@gmail.com',
      url='http://pypi.python.org/pypi/zeam.form.ztk',
      license='BSD',
      package_dir={'': 'src'},
      namespace_packages=['zeam', 'zeam.form'],
      packages=find_packages('src'),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'grokcore.component',
          'grokcore.view',
          'grokcore.security',
          'grokcore.chameleon >= 1.0.3',
          'setuptools',
          'zeam.form.base >= 1.2.3',
          'zope.component',
          'zope.container',
          'zope.event',
          'zope.i18n',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.lifecycleevent',
          'zope.publisher',
          'zope.schema >= 3.8',
          'zope.traversing',
        ],
      tests_require = tests_require,
      extras_require = {'test': tests_require,
                        'email': ['z3c.schema'],
                        'fanstatic': ['fanstatic',
                                      'js.jquery',
                                      'zeam.jsontemplate']},
      entry_points="""
      # -*- Entry points: -*-
      [zeam.form.components]
      default = zeam.form.ztk.fields:registerDefault
      bool = zeam.form.ztk.widgets.bool:register
      choice = zeam.form.ztk.widgets.choice:register
      collection = zeam.form.ztk.widgets.collection:register
      date = zeam.form.ztk.widgets.date:register
      email = zeam.form.ztk.widgets.email:register
      number = zeam.form.ztk.widgets.number:register
      object = zeam.form.ztk.widgets.object:register
      password = zeam.form.ztk.widgets.password:register
      text = zeam.form.ztk.widgets.text:register
      textline = zeam.form.ztk.widgets.textline:register
      time = zeam.form.ztk.widgets.time:register
      uri = zeam.form.ztk.widgets.uri:register
      [fanstatic.libraries]
      zeam.form.ztk.widgets = zeam.form.ztk.widgets.collection:library
      """,
      )



# -*- coding: utf-8 -*-

from zope.interface import alsoProvides
from zeam.form.base.datamanager import BaseDataManager


def makeGenericAdaptiveDataManager(*fields):

    class GenericAdaptiveDataManager(BaseDataManager):
        """A data manager that adapt its content to an interface
        before doing anything.
        """
        def __init__(self, content):
            self.fields = {}
            self.adapters = {}
            for field in fields:
                self.fields[field.identifier] = field.interface
                if (not field.interface.providedBy(content) and
                    not field.interface in self.adapters):
                    alsoProvides(self, field.interface)
                    self.adapters[field.interface] = field.interface(content)
            self.content = content

        def _getAdapter(self, identifier):
            if not identifier in self.fields:
                return self.content
            return self.adapters.get(self.fields.get(identifier))

        def get(self, identifier):
            content = self._getAdapter(identifier)
            if content is not None:
                try:
                    return getattr(content, identifier)
                except AttributeError:
                    pass
            raise KeyError(identifier)

        def set(self, identifier, value):
            content = self._getAdapter(identifier)
            if content is None:
                raise KeyError(identifier)
            setattr(content, identifier, value)

        def delete(self, identifier):
            content = self._getAdapter(identifier)
            if content is not None:
                try:
                    return delattr(content, identifier)
                except AttributeError:
                    pass
            raise KeyError(identifier)


    return GenericAdaptiveDataManager



# -*- coding: utf-8 -*-

from zeam.form.base import Action
from zeam.form.base.datamanager import ObjectDataManager
from zeam.form.base.markers import NO_CHANGE, SUCCESS, FAILURE
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.i18nmessageid import MessageFactory
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent

_ = MessageFactory("zeam.form.base")


class CancelAction(Action):
    """Cancel the current form and return on the default content view.
    """

    def __call__(self, form):
        form.redirect(form.url())


class EditAction(Action):
    """Edit the form content using the form fields.
    """

    def applyData(self, form, content, data):
        for field in form.fields:
            value = data.getWithDefault(field.identifier, default=NO_CHANGE)
            if value is not NO_CHANGE:
                content.set(field.identifier, value)

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return FAILURE

        content = form.getContentData()
        self.applyData(form, content, data)
        notify(ObjectModifiedEvent(content.content))
        form.status = _(u"Modification saved.")
        return SUCCESS


class AddAction(EditAction):
    """Add a new content in the form content, saving the form fields
    on the newly created content.
    """
    fieldName = 'title'

    def __init__(self, title, factory):
        super(AddAction, self).__init__(title)
        self.factory = factory

    def create(self, form, data):
        content = self.factory()
        self.applyData(form, ObjectDataManager(content), data)
        notify(ObjectCreatedEvent(content))
        return content

    def add(self, form, content, data):
        container = form.getContent()
        default_name = None
        if self.fieldName is not None:
            default_name = getattr(content, self.fieldName)

        chooser = INameChooser(container)
        name = chooser.chooseName(default_name, content)
        container[name] = content

    def nextURL(self, form, content):
        return form.url(content)

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return FAILURE

        content = self.create(form, data)
        self.add(form, content, data)
        form.redirect(self.nextURL(form, content))
        return SUCCESS



"""
We are going to define a simple form with an action and two fields
coming from a Zope interface.

We put our example in a separate file, since the configure.zcml of
zeam.form needs to be loaded in order to be able to create the fields,
which is no the case when the tests are collected.

Let's grok our example:

  >>> from zeam.form.ztk.testing import grok
  >>> grok('zeam.form.ztk.ftests.forms.ztkform_fixture')

We can now lookup our form by the name of its class:

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

  >>> from zeam.form.ztk.ftests.forms.ztkform_fixture import Person
  >>> context = Person()

  >>> from zope import component
  >>> form = component.getMultiAdapter(
  ...     (context, request), name='personform')
  >>> form
  <zeam.form.ztk.ftests.forms.ztkform_fixture.PersonForm object at ...>

  >>> len(form.actions)
  1
  >>> len(form.fields)
  2


Integration test
----------------

Let's try to take a browser and submit that form:

  >>> root = getRootFolder()
  >>> root['person'] = context

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

We can access the form, fill it and submit it:

  >>> browser.open('http://localhost/person/personform')
  >>> namefield = browser.getControl('Person name')
  >>> namefield
  <Control name='form.field.name' type='text'>
  >>> namefield.value = 'Arthur Sanderman'

  >>> agefield = browser.getControl('Person age')
  >>> agefield
  <Control name='form.field.age' type='number'>
  >>> agefield.value = '42'

  >>> action = browser.getControl('Send')
  >>> action
  <SubmitControl name='form.action.send' type='submit'>

  >>> action.click()
  >>> 'We sent Arthur Sanderman, age 42' in browser.contents
  True

"""




from zeam.form.base.fields import Field
from zeam.form.base.widgets import FieldWidget
from zeam.form.ztk.fields import registerSchemaField
from zeam.form.base.markers import Marker, NO_VALUE

from grokcore import component as grok
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface

_ = MessageFactory("zeam.form.base")
rfc822_specials = '()<>@,;:\\"[]'

def isMailAddress(addr):
    """Returns True if the email address is valid and False if not."""

    # First we validate the name portion (name@domain)
    c = 0
    while c < len(addr):
        if addr[c] == '@':
            break
        # Make sure there are only ASCII characters
        if ord(addr[c]) <= 32 or ord(addr[c]) >= 127:
            return False
        # A RFC-822 address cannot contain certain ASCII characters
        if addr[c] in rfc822_specials:
            return False
        c = c + 1

    # check whether we have any input and that the name did not end with a dot
    if not c or addr[c - 1] == '.':
        return False

    # check also starting and ending dots in (name@domain)
    if addr.startswith('.') or addr.endswith('.'):
        return False

    # Next we validate the domain portion (name@domain)
    domain = c = c + 1
    # Ensure that the domain is not empty (name@)
    if domain >= len(addr):
        return False
    count = 0
    while c < len(addr):
        # Make sure that domain does not end with a dot or has two dots in a row
        if addr[c] == '.':
            if c == domain or addr[c - 1] == '.':
                return False
            count = count + 1
        # Make sure there are only ASCII characters
        if ord(addr[c]) <= 32 or ord(addr[c]) >= 127:
            return False
        # A RFC-822 address cannot contain certain ASCII characters
        if addr[c] in rfc822_specials:
            return False
        c = c + 1
    if count >= 1:
        return True
    return False


class EmailField(Field):

    def __init__(self, title,
                 minLength=0,
                 maxLength=None,
                 **options):
        super(EmailField, self).__init__(title, **options)
        self.minLength = minLength
        self.maxLength = maxLength

    def isEmpty(self, value):
        return value is NO_VALUE or not len(value)

    def validate(self, value, form):
        error = super(EmailField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker) and len(value):
            assert isinstance(value, basestring)
            if not isMailAddress(value):
                return _(u"This email is not valid.")
            if self.minLength and len(value) < self.minLength:
                return _(u"This email is too short.")
            if self.maxLength and len(value) > self.maxLength:
                return _(u"This email is too long.")
        return None


class EmailWidget(FieldWidget):
    grok.adapts(EmailField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-email']
    defaultHtmlAttributes = set(['readonly', 'required', 'autocomplete',
                                 'maxlength', 'pattern', 'placeholder',
                                 'size', 'style'])


class EmailDisplayWidget(FieldWidget):
    grok.adapts(EmailField, Interface, Interface)
    grok.name('display')


def EmailSchemaFactory(schema):
    field = EmailField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        minLength=schema.min_length,
        maxLength=schema.max_length,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultValue=schema.default or NO_VALUE)
    return field


try:
    from z3c.schema.email.interfaces import IRFC822MailAddress

    def register():
        registerSchemaField(EmailSchemaFactory, IRFC822MailAddress)

except ImportError:

    def register():
        pass



# -*- coding: utf-8 -*-

import datetime

from zeam.form.base.markers import NO_VALUE, Marker
from zeam.form.base.widgets import FieldWidget, DisplayFieldWidget
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import Field, registerSchemaField

from grokcore import component as grok
from zope.i18n.format import DateTimeParseError
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


class TimeField(Field):
    """A time field.
    """
    valueLength = 'short'

    def __init__(self, title,
                 min=None,
                 max=None,
                 **options):
        super(TimeField, self).__init__(title, **options)
        self.min = min
        self.max = max

    def getFormatter(self, form):
        return form.request.locale.dates.getFormatter('time', self.valueLength)

    def validate(self, value, form):
        error = super(TimeField, self).validate(value, form)
        if error is not None:
            return error
        if not isinstance(value, Marker):
            assert isinstance(value, datetime.time)
            if self.min is not None and value < self.min:
                formatter = self.getFormatter(form)
                return _(u"This time is before ${not_before}.",
                         dict(not_before=formatter.format(self.min)))
            if self.max is not None and value > self.max:
                formatter = self.getFormatter(form)
                return _(u"This time is after ${not_after}.",
                         dict(not_after=formatter.format(self.max)))
        return None

# BBB
TimeSchemaField = TimeField


class TimeFieldWidget(FieldWidget):
    grok.adapts(TimeField, Interface, Interface)
    defaultHtmlClass = ['field', 'field-time']

    def valueToUnicode(self, value):
        formatter = self.component.getFormatter(self.form)
        return formatter.format(value)


class TimeWidgetExtractor(WidgetExtractor):
    grok.adapts(TimeField, Interface, Interface)

    def extract(self):
        value, error = super(TimeWidgetExtractor, self).extract()
        if value is not NO_VALUE:
            formatter = self.component.getFormatter(self.form)
            try:
                value = formatter.parse(value)
            except (ValueError, DateTimeParseError), error:
                return None, str(error)
        return value, error


class TimeFieldDisplayWidget(DisplayFieldWidget):
    grok.adapts(TimeField, Interface, Interface)

    def valueToUnicode(self, value):
        formatter = self.component.getFormatter(self.form)
        return formatter.format(value)


def TimeSchemaFactory(schema):
    field = TimeField(
        schema.title or None,
        identifier=schema.__name__,
        description=schema.description,
        required=schema.required,
        readonly=schema.readonly,
        min=schema.min,
        max=schema.max,
        interface=schema.interface,
        constrainValue=schema.constraint,
        defaultFactory=schema.defaultFactory,
        defaultValue=schema.__dict__['default'] or NO_VALUE)
    return field

def register():
    registerSchemaField(TimeSchemaFactory, schema_interfaces.ITime)
